# класс для хранения характеристик геомертического положения взгляда/головы
class GeoMetrics:

    def __init__(self):
        #x_mean,x_std,x_min,x_max,x_25,x_50,x_75,y_mean,y_std,y_min,y_max,y_25,y_50,y_75
        self.x_mean = self.x_std = self.x_min = self.y_mean = self.y_std = self.y_min = 0
        self.x_max = self.x_25 = self.x_50 = self.y_max = self.y_25 = self.y_50 = 0



# модуль анализа входных данных, выделяющий количественные характеристики
# основной класс - ParsedData, объединяет методы вычисления, обработки и хранение данных о количественных характеристиках
from ..Input import read_csv_file
from ..Analise.ParsedData import ParsedData


# Парсинг по умолчанию
def parse_ff_tf(file_from, file_to):
    section = read_csv_file(file_from)
    metrics = ParsedData()
    metrics.parse(section, 3, 5)
    metrics.to_csv(file_from, file_to)

# Проверка данных на наличие пред-обработки / интерполяции
def check_interpolation():
    section = read_csv_file('../test_data1.csv')
    s = 0
    d = 0.001
    for i in range(1, len(section.positionData) - 1):
        if abs(section.positionData[i].get_distance(section.positionData[i - 1])
               - section.positionData[i].get_distance(section.positionData[i + 1])) < d:
            s += 1
    print("Suspicion: " + (s / len(section.positionData)).__str__())
    print(s)



import sys
from time import sleep
import pandas as pd
import pickle
from ..Analise.Names import eng_to_rus
from ..Input import read_csv_file
from ..Analise.ParsedData import ParsedData
from ..Evaluate.RandomForestEval import RandomForestEval
from ..Evaluate.DecisionTreeEval import DecisionTreeEval
from ..Evaluate.MLPEval import MLPEval
from ..Evaluate.BasicEval import BasicEval
from PyQt6.QtGui import QPixmap
from PyQt6.QtWidgets import *
from sklearn.metrics import (
    accuracy_score,
    f1_score
)

class EvalApp(QMainWindow):
    def __init__(self): # инициализация визуальных компонентов
        super().__init__()

        self.rb_base = QRadioButton('Базовый', self)
        self.rb_rfc = QRadioButton('Random Forest', self)
        self.rb_dtc = QRadioButton('Decision Tree', self)
        self.rb_mlp = QRadioButton('MLP', self)

        self.rb_group = QButtonGroup()
        self.rb_group.addButton(self.rb_base)
        self.rb_group.addButton(self.rb_rfc)
        self.rb_group.addButton(self.rb_dtc)
        self.rb_group.addButton(self.rb_mlp)
        self.rb_mlp.setChecked(True)
        self.model_name = 'Basic'
        self.rb_group.buttonClicked.connect(self.rb_click)

        self.data_list1 = []
        self.data_list2 = []
        self.data_list_text1 = QPlainTextEdit(self)
        self.data_list_text2 = QPlainTextEdit(self)
        self.model = None
        self.edu_list1 = None
        self.edu_list2 = None
        self.edu_list_text1 = QPlainTextEdit(self)
        self.edu_list_text2 = QPlainTextEdit(self)

        self.eval_list = None
        self.eval_list_text = QPlainTextEdit(self)

        self.res_list1 = ParsedData.get_df_null_row()
        self.res_list2 = ParsedData.get_df_null_row()
        self.res_list_text1 = QPlainTextEdit(self)
        self.res_list_text2 = QPlainTextEdit(self)
        self.arg_line = QLineEdit('1.0', self)
        self.file_size_line = QLineEdit('нет', self)

        self.corr = 0
        self.err = 0

        self.keys = ParsedData.get_df_null_row().drop(columns=['File'])
        self.keys.loc[0] = [0] * len(self.keys.columns)
        self.key_boxes = []
        
        self.ideal_min = ParsedData.get_df_null_row().drop(columns=['File'])
        self.ideal_max = ParsedData.get_df_null_row().drop(columns=['File'])
        self.f1_line = QLineEdit('0.0', self)
        # группы визуальных компонентов
        self.data_group = []
        self.edu_group = []
        self.eval_group = []
        self.res_group = []
        self.status_ = 0
        self.status_img = QLabel(self)
        self.status_text = QLabel(self)
        self.reset_status_btn = QPushButton('Сбросить', self)
        self.reset_status_btn.clicked.connect(lambda : self.set_status(0))

        self.setup_ui()

    def setup_ui(self): # инициализация с конкретными значениями
        self.setGeometry(300, 300, 1010, 650)
        self.setWindowTitle('Оценка данных')
        # группа загрузки данных
        data_label1 = QLabel('Бодрые', self)
        data_label1.setGeometry(80, 10, 200, 15)
        self.data_group.append(data_label1)

        data_files_btn = QPushButton('Загрузить данные бодрых', self)
        data_files_btn.setGeometry(10, 25, 200, 30)
        data_files_btn.clicked.connect(self.calc_data1)
        self.data_group.append(data_files_btn)

        self.data_list_text1.setStyleSheet("color: blue; background-color: white;")
        self.data_list_text1.setGeometry(10, 60, 200, 150)
        self.data_group.append(self.data_list_text1)

        data_label2 = QLabel('Усталые', self)
        data_label2.setGeometry(80, 210, 90, 15)
        self.data_group.append(data_label2)

        data_files_btn = QPushButton('Загрузить данные усталых', self)
        data_files_btn.setGeometry(10, 225, 200, 30)
        data_files_btn.clicked.connect(self.calc_data2)
        self.data_group.append(data_files_btn)

        self.data_list_text2.setStyleSheet("color: red; background-color: white;")
        self.data_list_text2.setGeometry(10, 260, 200, 150)
        self.data_group.append(self.data_list_text2)

        data_save_file_btn = QPushButton('Сохранить\nпосчитанные\nданные', self)
        data_save_file_btn.setGeometry(10, 420, 100, 60)
        data_save_file_btn.clicked.connect(self.save_data)
        self.data_group.append(data_save_file_btn)

        data_load_file_btn = QPushButton('Загрузить\nпосчитанные\nданные', self)
        data_load_file_btn.setGeometry(110, 420, 100, 60)
        data_load_file_btn.clicked.connect(self.load_data)
        self.data_group.append(data_load_file_btn)

        arg_label = QLabel('Размер области фиксации:', self)
        arg_label.setGeometry(10, 480, 150, 30)
        self.data_group.append(arg_label)
        self.arg_line.setGeometry(160, 480, 50, 30)
        self.data_group.append(self.arg_line)

        file_size_label = QLabel('Максимальный размер файла:', self)
        file_size_label.setGeometry(10, 520, 150, 30)
        self.data_group.append(file_size_label)
        self.file_size_line.setGeometry(160, 520, 50, 30)
        self.data_group.append(self.file_size_line)

    ##################################################################################################################
        # группа обучения моделей
        edu_files_btn = QPushButton('Разбить выборку для обучения', self)
        edu_files_btn.setGeometry(280, 10, 200, 30)
        edu_files_btn.clicked.connect(self.take_sample)
        self.edu_group.append(edu_files_btn)

        edu_label1 = QLabel('Бодрые', self)
        edu_label1.setGeometry(360, 40, 200, 15)
        self.edu_group.append(edu_label1)
        self.edu_list_text1.setStyleSheet("color: blue; background-color: white;")
        self.edu_list_text1.setGeometry(280, 60, 200, 150)
        self.edu_group.append(self.edu_list_text1)
        edu_label2 = QLabel('Усталые', self)
        edu_label2.setGeometry(360, 200, 90, 50)
        self.edu_group.append(edu_label2)
        self.edu_list_text2.setStyleSheet("color: red; background-color: white;")
        self.edu_list_text2.setGeometry(280, 240, 200, 150)
        self.edu_group.append(self.edu_list_text2)
        
        edu_btn = QPushButton('Обучение', self)
        edu_btn.setGeometry(280, 400, 200, 30)
        edu_btn.clicked.connect(self.do_edu)
        self.edu_group.append(edu_btn)

        edu_save_file_btn = QPushButton('Сохранить\n модель', self)
        edu_save_file_btn.setGeometry(280, 430, 100, 60)
        edu_save_file_btn.clicked.connect(self.save_edu_file)
        self.edu_group.append(edu_save_file_btn)
        edu_load_file_btn = QPushButton('Загрузить\n модель', self)
        edu_load_file_btn.setGeometry(380, 430, 100, 60)
        edu_load_file_btn.clicked.connect(self.load_edu_file)
        self.edu_group.append(edu_load_file_btn)

        self.crossvalid_box = QCheckBox('Кроссвалидация', self)
        self.crossvalid_box.setGeometry(280, 500, 200, 30)
        self.edu_group.append(self.crossvalid_box)
        self.normalize_box = QCheckBox('Нормализация', self)
        self.normalize_box.setGeometry(280, 540, 200, 30)
        self.edu_group.append(self.normalize_box)
        #crossvalid_box.clicked.connect(self.model_cross_validation)
        

    ##################################################################################################################
        # группа выполнения оценки
        eval_label = QLabel('На оценку', self)
        eval_label.setGeometry(550, 40, 200, 10)
        self.eval_list_text.setGeometry(550, 50, 200, 150)
        self.eval_group.append(eval_label)

        self.rb_base.move(550, 210)
        self.rb_rfc.move(550, 240)
        self.rb_dtc.move(550, 270)
        self.rb_mlp.move(550, 300)

        choose_all_btn = QPushButton('Всё', self)
        choose_all_btn.setGeometry(550, 370, 50, 30)
        choose_all_btn.clicked.connect(self.check_box_all)
        self.eval_group.append(choose_all_btn)

        choose_std_btn = QPushButton('Стандартный набор', self)
        choose_std_btn.setGeometry(610, 370, 200, 30)
        choose_std_btn.clicked.connect(self.check_box_std)
        self.eval_group.append(choose_std_btn)

        choose_find = QLineEdit('..', self)
        choose_find.setGeometry(820, 370, 180, 30)
        self.eval_group.append(choose_find)

        scroll_box = QScrollArea(self)
        scroll_box.setGeometry(550, 410, 430, 190)
        self.eval_group.append(scroll_box)

        self.scrollbar = QScrollBar(self)
        self.scrollbar.setGeometry(980, 410, 20, 190)
        self.scrollbar.sliderMoved.connect(self.scroll_check_box)
        self.eval_group.append(self.scrollbar)

        self.create_check_box(scroll_box)        

        res_files_btn = QPushButton('Выполнить оценку', self)
        res_files_btn.setGeometry(550, 10, 200, 30)
        res_files_btn.clicked.connect(self.do_eval)        
        self.eval_group.append(res_files_btn)

        res_label = QLabel('Результаты оценки', self)
        res_label.setGeometry(840, 10, 200, 15)
        self.eval_group.append(res_label)
        res_label1 = QLabel('Бодрые', self)
        res_label1.setGeometry(880, 30, 200, 10)
        self.eval_group.append(res_label1)
        self.res_list_text1.setStyleSheet("color: blue; background-color: white;")
        self.res_list_text1.setGeometry(800, 50, 200, 60)
        self.eval_group.append(self.res_list_text1)

        res_label2 = QLabel('Усталые', self)
        res_label2.setGeometry(880, 115, 90, 15)
        self.eval_group.append(res_label2)
        self.res_list_text2.setStyleSheet("color: red; background-color: white;")
        self.res_list_text2.setGeometry(800, 130, 200, 60)
        self.eval_group.append(self.res_list_text2)

        f1_label = QLabel('Точность оценки:',self)
        f1_label.setGeometry(800, 200, 100, 30)
        self.eval_group.append(f1_label)
        self.f1_line.setGeometry(900, 200, 100, 30)
        self.eval_group.append(self.f1_line)
        f1_label = QLabel('Кроссвалидация:',self)
        f1_label.setGeometry(800, 240, 100, 30)
        self.crossvalid_label = QLineEdit('', self)
        self.crossvalid_label.setGeometry(900, 240, 100, 30)
        self.eval_group.append(self.crossvalid_label)

        self.set_status(0)
        self.check_box_std()
        self.show()

    def rb_click(self, rb): # выбор используемых моделей оценки
        if rb.text() == 'Базовый':
            self.model_name = 'Basic'
        elif rb.text() == 'Random Forest':
            self.model_name = 'RFC'
        elif rb.text() == 'Decision Tree':
            self.model_name = 'DTC'
        elif rb.text() == 'MLP':
            self.model_name = 'MLP'

    def create_check_box(self, parent): # создание прокручиваемого окошка с выбором используемых численных характеристик
        x = 0
        y = 0
        w = parent.width()
        h = 15
        self.scr_area = QScrollArea(parent)
        for col in ParsedData.get_df_null_row().drop(columns='File').head(): # инициализация окошек выбора по заголовку данных от класса ParsedData
            chbx = QCheckBox(eng_to_rus(str(col)), self.scr_area)
            chbx.setChecked(self.keys[col].iloc(0) == 1)
            chbx.setGeometry(x, y, w, h)
            chbx.clicked.connect(self.check_box_click)
            self.key_boxes.append(chbx)
            y += h 
        self.scr_area.resize(parent.width(), y)
        self.scrollbar.setMaximum(y - parent.height() + 15)
        self.scrollbar.setMinimum(-8)
    
    def scroll_check_box(self): # реализация прокрутки
        value = self.scrollbar.value()
        self.scr_area.move(0, -value)

    def check_box_click(self): # переназначение используемых численных характеристик
        i = 0
        for col in self.keys.head():
            self.keys[col][0] = 1 if self.key_boxes[i].isChecked() else 0
            i += 1
    
    def check_box_all(self): # выбор ВСЕХ численных характеристик
        i = 0
        fl = not self.key_boxes[i].isChecked()
        for col in self.keys.head():
            self.keys[col][0] = 1 if fl else 0
            self.key_boxes[i].setChecked(fl)
            i += 1
    
    def check_box_std(self): # выбор ВСЕХ численных характеристик
        '''
        Текущий стандартный набор:
        средняя кривизна траектории движения взгляда;  +
        минимальная кривизна траектории движения взгляда; +
        минимальная длина саккады; +
        доля времени, проведённого в фиксациях короче 150 мс;  +
        доля фиксаций короче 150 мс.  +
        средняя скорость внутри области фиксации, °/сек; +
        максимальная скорость внутри области фиксации, °/сек + 
        '''
        std = {'Average Curve', 'Min Curve', 'Min Saccade Time', 'Average Fixation Speed', 'Max Fixation Speed', "% of Fixations < 150 ms", "Fixation time < 150 ms, per time"}
        # вариации наборов, можно добавить свои
        # уточнить русское-английское название характеристик можно в файле Names в модуле Analise
        #std = {'x_mean', 'x_std', 'x_min', 'x_max', 'x_25', 'x_50', 'x_75', 'y_mean', 'y_std', 'y_min', 'y_max', 'y_25', 'y_50', 'y_75', 'Saccades with amplitude < 6 degrees, per minute', 'Max Curve', 'Fixation time > 150 ms', 'Fixation time > 180 ms', '% of Fixations < 150 ms', '% of Fixations > 150 ms', '% of Fixations > 900 ms', '% of Fixations < 180 ms', '% of Fixations > 180 ms', 'Fixation time < 150 ms, per time', 'Fixation time > 150 ms, per time', '% of Fixations > 150 ms, per minute', '% of Fixations < 180 ms, per minute', 'Min Speed', 'Max Speed', 'Average Speed in interval (1s)', 'Max Speed in interval (1s)', 'Average Fixation Speed', 'Max Fixation Speed', 'Average Saccade Length', 'Min Saccade Length', 'Max Saccade Length'}
        #std = {'x_mean', 'x_std', 'x_min', 'x_max', 'x_25', 'x_50', 'x_75', 'y_mean', 'y_std', 'y_min', 'y_max', 'y_25', 'y_50', 'y_75'}
        #std = {'x_mean', 'y_mean', 'Average Curve', 'Min Curve', 'Min Saccade Time', 'Average Fixation Speed', "% of Fixations < 150 ms, per minute", 'Average Fixation Speed, < 150ms'}
        i = 0
        for col in self.keys.head():
            self.keys[col][0] = 1 if col in std else 0
            self.key_boxes[i].setChecked(col in std)
            i += 1

    def calc_df(self, files): # рассчёт характеристик выбранных файлов
        arg = float(self.arg_line.text())
        data_frame = ParsedData.get_df_null_row()
        file_size = -1  if self.file_size_line.text() == 'нет' else float(self.file_size_line.text())
        for line in files:
            p = 1
            print(file_size)
            for section in read_csv_file(line).split(file_size):
                metrics = ParsedData()
                metrics.parse(section, 2, arg) # по умолчанию для разметки используется метод определения фиксаций по абсолютному расстоянию, можно переключить
                metrics.calc_metrics()
                row = metrics.get_df_row(line.split('/')[-1] + ('' if file_size == -1 else 'part '+str(p)))
                p += 1
                data_frame = pd.concat([data_frame, row], ignore_index=True)
        return data_frame

    def calc_data1(self): # загрузка и рассчёт "бодрых" записей
        csv_path, _ = QFileDialog.getOpenFileNames(caption='Загрузите Бодрых')
        self.data_list1 = self.calc_df(csv_path)
        self.data_list_text1.setPlainText('')
        for line in self.data_list1['File']:
            self.data_list_text1.setPlainText(self.data_list_text1.toPlainText() + line + '\n')
    
    def calc_data2(self): # загрузка и рассчёт "уставших" записей
        csv_path, _ = QFileDialog.getOpenFileNames(caption='Загрузите Уставших')
        self.data_list2 = self.calc_df(csv_path)
        self.data_list_text2.setPlainText('')
        for line in self.data_list2['File']:
            self.data_list_text2.setPlainText(self.data_list_text2.toPlainText() + line + '\n')
        self.set_status(1) # обновление статуса процесса

    def save_data(self): # сохранение посчитанных данных
        file_name = QFileDialog.getSaveFileName(directory='_Data')
        print(file_name)
        self.data_list1.to_csv(file_name[0] + ' Бодрые.csv', sep=';', index=False)
        self.data_list2.to_csv(file_name[0] + ' Усталые.csv', sep=';', index=False)

    def load_data(self): # загрузка посчитанных данных
        file_name = QFileDialog.getOpenFileName(directory='_Data', caption='Бодрые')
        self.data_list1 = pd.read_csv(file_name[0], sep=';')
        file_name = QFileDialog.getOpenFileName(directory='_Data', caption='Уставшие')
        self.data_list2 = pd.read_csv(file_name[0], sep=';')
        self.data_list_text1.setPlainText('')
        for line in self.data_list1['File']:
            self.data_list_text1.setPlainText(self.data_list_text1.toPlainText() + str(line) + '\n')
        self.data_list_text2.setPlainText('')
        for line in self.data_list2['File']:
            self.data_list_text2.setPlainText(self.data_list_text2.toPlainText() + str(line) + '\n')
        self.set_status(1) # обновление статуса процесса

    def take_sample(self): # генерация выборки из данных
        print('load edu button click')
        self.edu_list1 = self.data_list1.sample(n=int(0.8*len(self.data_list1))) # выборки генерируются раномерно
        self.edu_list2 = self.data_list2.sample(n=int(0.8*len(self.data_list2))) # по 80% бодрых и уставших записей для обучения, оставшиеся - для проверки
        ev1 = pd.concat([self.data_list1, self.edu_list1]).drop_duplicates(keep=False)
        ev2 = pd.concat([self.data_list2, self.edu_list2]).drop_duplicates(keep=False)
        self.eval_list = pd.concat([ev1, ev2], ignore_index=True)
        '''
        # проверочный вывод в консоль
        print(len(self.data_list1))
        print(len(self.data_list2))
        print(len(self.edu_list1))
        print(len(self.edu_list2))
        print(len(self.eval_list))
        '''
        self.edu_list_text1.setPlainText('')
        for _, row in self.edu_list1.iterrows():
            self.edu_list_text1.setPlainText(self.edu_list_text1.toPlainText() + str(row['File']) + '\n')            
        self.edu_list_text2.setPlainText('')
        for _, row in self.edu_list2.iterrows():
            self.edu_list_text2.setPlainText(self.edu_list_text2.toPlainText() + str(row['File']) + '\n')

        self.eval_list_text.setPlainText('')
        for _, row in self.eval_list.iterrows():
            self.eval_list_text.setPlainText(self.eval_list_text.toPlainText() + str(row['File']) + '\n')                 

    def save_edu_file(self): # сохранение обученной модели
        pkl_filename = QFileDialog.getSaveFileName(directory='_Models')[0] # выбор имени
        with open(pkl_filename, 'wb') as file: 
            pickle.dump(self.model, file) 

    def load_edu_file(self, pkl_filename ='model.pkl'): # загрузка обученной модели
        pkl_filename = QFileDialog.getOpenFileName(directory='_Models', caption='Загрузите модель')[0] # выбор файла
        if pkl_filename == '':
            return
        with open(pkl_filename, 'rb') as file: # загрузки и определение типа модели
            self.model = pickle.load(file)
            if self.model.get_name == 'Basic':
                self.rb_base.setChecked(True)
            if self.model.get_name == 'MLP':
                self.rb_mlp.setChecked(True)
            elif self.model.get_name == 'DTC':
                self.rb_dtc.setChecked(True)
            elif self.model.get_name == 'RFC':
                self.rb_rfc.setChecked(True)
        self.set_status(2) # переключение статуса процесса

    def do_eval(self): # выполнение оценки
        print('eval')
        self.res_list1 = ParsedData.get_df_null_row()
        self.res_list2 = ParsedData.get_df_null_row()
        file_data = self.eval_list.drop(columns='File')
    
        for col in self.keys.columns:
            if self.keys[col][0] == 0 and file_data.columns.__contains__(col):
                file_data = file_data.drop(columns=[col])
        tired = self.model.evaluate(file_data)
        for i in range(len(tired)):
            if not tired[i]:
                self.res_list1.loc[len(self.res_list1)] = self.eval_list['File'][i]
            else:
                self.res_list2.loc[len(self.res_list2)] = self.eval_list['File'][i]
        
        self.corr = 0
        self.err = 0
        print(self.res_list1)
        # подсчёт верных и неверных оценок (по соответствию с изначально загруженными данными)
        for _, row in self.res_list1.iterrows():
            if self.data_list_text1.toPlainText().__contains__(row['File']) : # gjghfdbnm
                self.corr += 1
            else:
                self.err += 1
        for _, row in self.res_list2.iterrows():
            if self.data_list_text2.toPlainText().__contains__(row['File']): #
                self.corr += 1
            else:
                self.err += 1
        #print('Corr: ' + str(self.corr) + '  Err: ' + str(self.err))
        corr_sum = self.corr / (self.corr + self.err)
        self.f1_line.setText(format(corr_sum, '.2f'))
        self.print_result()

    def print_result(self): # вывод результатов
        print('print result')
        self.res_list_text1.setPlainText('')
        for _, row in self.res_list1.iterrows():
            self.res_list_text1.setPlainText(self.res_list_text1.toPlainText() + str(row['File']) + '\n')
        self.res_list_text2.setPlainText('')
        for _, row in self.res_list2.iterrows():
            self.res_list_text2.setPlainText(self.res_list_text2.toPlainText() + str(row['File']) + '\n')
        self.output_result() # вывод результатов оценки в файл (можно закомментировать, если не требуется)
        self.set_status(3) # переключение статуса процесса

    def output_result(self): # вывод результатов оценки в файл
        df = pd.DataFrame({ 'File' : [], 'Eval' : [] })
        for _, row in self.res_list1.iterrows():
            df.loc[-1] = [row['File'], 1]
            df.index = df.index + 1
        for _, row in self.res_list2.iterrows():
            df.loc[-1] = [row['File'], 0]
            df.index = df.index + 1
        df.to_csv('output.csv', sep=';')

    def do_edu(self, redu=False): # обучение выбранной модели
        # выделение данных характеристик для обучения и для проверки
        train_X = pd.concat([self.edu_list1.drop(columns = ['File']), self.edu_list2.drop(columns = ['File'])])
        test_X = self.eval_list.drop(columns = ['File'])
        for col in self.keys.columns:
            if self.keys[col][0] == 0 and col in train_X.columns:
                train_X = train_X.drop(columns=[col])
                test_X = test_X.drop(columns=[col])
        
        if self.normalize_box.isChecked(): # выполнение нормализации, если выбран флажок
            gen = pd.concat([train_X, test_X], ignore_index=True)
            train_X = (train_X - gen.mean())/gen.std()
            test_X = (test_X - gen.mean())/gen.std()
        # выделение данных оценки для обучения и для проверки
        train_Y = pd.DataFrame({ 
            'fatique' : [1] * len(self.edu_list1) + [0] * len(self.edu_list2)
        })
        test_Y = pd.DataFrame({
            'fatique' : [1] * (len(self.data_list1) - len(self.edu_list1)) + [0] * (len(self.data_list2) - len(self.edu_list2))
        })

        if redu: # вариант с переобучением модели (с сохранением параметров) на новых данных
            self.model.redu(train_X, train_Y, test_X, test_Y)   
        else: # обучение модели с нуля
            if self.model_name == 'Basic':
                self.model = BasicEval()
            if self.model_name == 'RFC':
                self.model = RandomForestEval()
            elif self.model_name == 'DTC':
                self.model = DecisionTreeEval()
            elif self.model_name == 'MLP':
                self.model = MLPEval()
            if self.crossvalid_box.isChecked(): # вариация обучения с использование кроссвалидации
                # формирование общей выборки
                data_X = pd.concat([self.data_list1.drop(columns = ['File']), self.data_list2.drop(columns = ['File'])], ignore_index=True)
                if self.normalize_box.isChecked(): 
                    data_X = (data_X - data_X.mean())/data_X.std() # нормализация
                data_X.dropna()
                data_Y = pd.DataFrame({
                        'fatique' : [1] * len(self.data_list1) + [0] * len(self.data_list2)
                })
                data = pd.concat([data_X, data_Y], axis=1)
                data = data.sample(frac=1) # перемешивание выборки
                data_Y = data['fatique']
                data_X = data.drop(columns=['fatique'])
                self.model.cross_edu(data_X, data_Y) # выполнение обучения с кроссвалидацией (не работает для алгоритмической модели)
                # вывод результатов обучения
                self.f1_line.setText('F1: ' + format(self.model.f1,'.3f') + 
                                    ' Acc: ' + format(self.model.acc, '.3f'))
                self.crossvalid_label.setText('F1: ' + format(self.model.cross_f1,'.3f') + 
                                    ' Acc: ' + format(self.model.cross_acc, '.3f'))
            else:
                # обучение без кроссвалидации
                self.model.edu(train_X, train_Y, test_X, test_Y)

    def set_status(self, status): # установка статуса процесса (облегчает ориентацию в визуальном интерфейсе, отображает текущую стадию обработки данных)
        self.status = status
        self.status_text.setGeometry(0, self.height()-30, 3000, 30)
        self.reset_status_btn.setGeometry(self.width()-100, self.height()-30, 100, 30)
        self.status_img.setGeometry(0, self.height()-30, 3000, 30)

        '''
        # опционально можно блокировать группы визуальных компонентов в зависимости от текущей стадии процесса
        for widget in self.data_group:
            widget.setEnabled(status == 0)
        for widget in self.edu_group:
            widget.setEnabled(status == 1)
        for widget in self.eval_group:
            widget.setEnabled(status > 1)
        for widget in self.res_group:
            widget.setEnabled(status == 3)
        '''
        # текстовое и цветовое сопровождение
        if status == 0:
            self.status_text.setText('  Загрузите данные для обработки')
            self.status_img.setPixmap(QPixmap('UI/Red.png'))
        if status == 1:
            self.status_text.setText('  Обучите модель или загрузите уже обученную')
            self.status_img.setPixmap(QPixmap('UI/Orange.png'))
        if status == 2:
            self.status_text.setText('  Программа готова к запуску оценки')
            self.status_img.setPixmap(QPixmap('UI/Yellow.png'))
        if status == 3:
            self.status_text.setText('  Оценка выполнена')
            self.status_img.setPixmap(QPixmap('UI/Green.png'))

# инициализация приложения через мейн процесс
if __name__ == '__main__':
    app = QApplication(sys.argv)
    sleep(10)
    ex = EvalApp()
    sys.exit(app.exec_())



from time import sleep
import sys
from PyQt6.QtGui import QPainter, QColor
from ..Input import read_csv_file
from ..Analise.ParsedData import ParsedData
from PyQt6.QtWidgets import QMainWindow, QPushButton, QApplication, QLabel, QFileDialog, QLineEdit, QRadioButton, \
    QButtonGroup

class Example(QMainWindow):

    def __init__(self):  # инициализация визуальных компонентов
        super().__init__()

        self.rb_speed = QRadioButton('Speed', self)
        self.rb_speed.setChecked(True)
        self.I = 0
        self.rb_area = QRadioButton('Area', self)
        self.rb_dist = QRadioButton('Dist', self)

        self.rb_group = QButtonGroup()
        self.rb_group.addButton(self.rb_speed)
        self.rb_group.addButton(self.rb_area)
        self.rb_group.addButton(self.rb_dist)
        self.rb_group.buttonClicked.connect(self.rb_click)

        self.method_label = QLabel('Определение фиксаций\nпо скорости', self)
        self.method_label.adjustSize()
        self.file_line = QLineEdit(self)
        self.arg_label = QLabel("Порог скорости", self)
        self.arg_line = QLineEdit(self)

        self.arg_0_line = QLineEdit(self)
        self.arg_1_line = QLineEdit(self)
        self.arg_step_line = QLineEdit(self)
        self.initUI()

    def rb_click(self, rb): # обработка выбора метода размеметки
        if rb.text() == 'Speed':
            self.I = 0
            self.method_label.setText('Определение фиксаций\nпо скорости')
            self.arg_line.setText('220') # установка значений аргумента по умолчанию
            self.arg_label.setText('Порог скорости')
        if rb.text() == 'Area':
            self.I = 1
            self.method_label.setText('Определение фиксаций\nпо области')
            self.arg_line.setText('5') # установка значений аргумента по умолчанию
            self.arg_label.setText('Диаметр области фиксации')
        if rb.text() == 'Dist':
            self.I = 2
            self.method_label.setText('Определение фиксаций\nпо абсолютному расстоянию')
            self.arg_line.setText('5') # установка значений аргумента по умолчанию
            self.arg_label.setText('Максимальное расстояние')
        self.method_label.adjustSize()
        self.arg_label.adjustSize()

    def initUI(self): # инициализация интерфейса с конкретными значениями
        self.rb_speed.move(20, 60)
        self.rb_area.move(20, 80)
        self.rb_dist.move(20, 100)
        self.method_label.move(20, 20)
        # устаревшая часть - UI для указания центральной области
        # rad_label = QLabel('Диаметр центральной области', self)
        # rad_label.move(200, 20)
        # rad_label.adjustSize()
        # self.rad_line.setText('5')
        # self.rad_line.setGeometry(200, 40, 100, 20)

        # ввод аргумента (для метода разметки)
        self.arg_line.setGeometry(200, 80, 100, 20)
        self.arg_line.setText('220')
        self.arg_label.move(200, 60)
        self.arg_label.adjustSize()

        # ввод аргументов с перебором (для метода разметки)
        self.arg_0_line.setGeometry(20, 160, 30, 20)
        self.arg_0_line.setText('1')
        self.arg_1_line.setGeometry(50, 160, 30, 20)
        self.arg_1_line.setText('2.5')
        self.arg_step_line.setGeometry(80, 160, 30, 20)
        self.arg_step_line.setText('0.2')
        # зона загрузки файлов, запуска рассчётов
        load_label = QLabel('Расчёт делается\nсразу по\nвыбору файла', self)
        load_label.move(380, 20)
        load_label.adjustSize()
        load_btn = QPushButton('Выбрать файлы', self)
        load_btn.setGeometry(380, 80, 100, 30)
        load_btn.clicked.connect(self.csvLoadButton)
        file_label = QLabel('Output file', self)
        file_label.move(380, 110)
        self.file_line.setText('masterfile.csv')
        self.file_line.setGeometry(380, 140, 100, 20)

        ppi_label = QLabel('Определение PPI', self)
        ppi_label.adjustSize()
        ppi_label.move(220, 140)
        load_btn = QPushButton('Выбрать файлы PPI', self)
        load_btn.setGeometry(220, 160, 120, 30)
        load_btn.clicked.connect(self.ppiLoadButton)

        find_btn = QPushButton('Счёт в \n интервале', self)
        find_btn.setGeometry(120, 145, 65, 40)
        find_btn.clicked.connect(self.findSec)

        pupil_btn = QPushButton('Файлы pupillabs', self)
        pupil_btn.setGeometry(480, 80, 100, 30)
        pupil_btn.clicked.connect(self.pupilButton)

        self.statusBar()
        self.setGeometry(300, 300, 600, 200)
        self.setWindowTitle('Вычисление количественных характеристик')
        self.show()

    def ppiLoadButton(self): # загрузка данных о преимпульсном ингибировании 
        print('ppi button click')
        ppi_path, _ = QFileDialog.getOpenFileNames()
        if len(ppi_path) == 0:
            return
        print('files chosen')
        for file in ppi_path:
            if not file.__contains__('.txt'):
                print('cant read file ' + file)
                continue
            print('reading file ' + file)
            lines = open(file).readlines()
            ParsedData.ppi_to_xls(file, lines, self.file_line.text())

    def pupilButton(self): # загрузка файлов, полученных от PupilLabs (сервис поставщика окулографа)
        print('pupil button click')
        csv_path, _ = QFileDialog.getOpenFileNames()
        pupil_path, _ = QFileDialog.getOpenFileNames()

        if len(csv_path) == 0:
            return
        print('files chosen')
        if len(pupil_path) != len(csv_path):
            print('number of file are not matching')
            return
        for i in range(0, len(csv_path)):
            if not csv_path[i].__contains__('.csv'):
                print('cant read file ' + csv_path[i])
                continue
            if not pupil_path[i].__contains__('.csv'):
                print('cant read file ' + pupil_path[i])
                continue
            print('reading file ' + csv_path[i] + ' with data ' + pupil_path[i])
            section = read_csv_file(csv_path[i])
            metrics = ParsedData()
            metrics.parse_pupil(section, pupil_path[i])
            metrics.calc_metrics()
            metrics.to_xls(csv_path[i].split('/')[-1], self.file_line.text())

    def csvLoadButton(self): # загрузка файлов на обработку
        print('load button click')
        csv_path, _ = QFileDialog.getOpenFileNames() # выбор файлов
        if len(csv_path) == 0:
            return
        print('files chosen')
        arg = float(self.arg_line.text())
        # rad = float(self.rad_line.text()) / 2
        for file in csv_path: # перебор выбранных файлов
            if not file.__contains__('.csv'):
                print('cant read file ' + file)
                continue
            print('reading file ' + file)
            section = read_csv_file(file) # считывание сырых данных
            print('parsing file by ' + self.arg_label.text() + ' = ' + self.arg_line.text())
            metrics = ParsedData() 
            metrics.parse(section, self.I, arg) # разметка данных выбранным методом
            metrics.calc_metrics() # рассчёт метрик
            metrics.to_csv(file.split('/')[-1], self.file_line.text()) # вывод результата

    def findSec(self): # загрузка файлов на обработку с перебором аргумекта
        print('find sec click')
        csv_path, _ = QFileDialog.getOpenFileNames() # выбор файлов
        if len(csv_path) == 0:
            return
        print('files chosen')
        arg0 = float(self.arg_0_line.text()) # выделение указанных границ и шага аргумента
        arg1 = float(self.arg_1_line.text())
        step = float(self.arg_step_line.text())
        # rad = float(self.rad_line.text()) / 2
#        diff_list = []

        total = len(csv_path) * ((arg1 - arg0)/step).__int__()
        current = 0
        for ii in range(0, ((arg1 - arg0)/step).__int__()):
            arg = arg0 + ii * step
#            lines = []
            for file in csv_path:
                if not file.__contains__('.csv'):
                    print('cant read file ' + file)
                    continue
                print('reading file ' + file)
                section = read_csv_file(file) # считывание сырых данных
                metrics = ParsedData() 
                metrics.parse(section, self.I, arg) # разметка данных выбранным методом
                metrics.calc_metrics() # рассчёт метрик
                metrics.to_csv(file.split('/')[-1], 'Summary.csv') # вывод
                current += 1
                print( (float(current) / float(total) * 100.0).__str__() + " % of progress")

# инициализация приложения через мейн процесс
if __name__ == '__main__':
    app = QApplication(sys.argv)
    sleep(1)
    ex = Example()
    sys.exit(app.exec_())



import os
import numpy as np
import cv2 # для видеообработки требуется также библиоетка cv2, не включённая в основной список зависимостей
from math import pi
import datetime
import os.path
import statistics as st
from scipy import ndimage as ndi


class SceneProcessor:
    EyetrackerHorizontalFOV = 82
    EyetrackerFrameWidth = -1
    EyetrackerFrameHeight = -1
    EyetrackerFPS = 30
    _trajectory_x = []  # координаты по x и
    _trajectory_y = []  # y
    _trajectory_r = []  # Наклон головы
    _confidence = []    # Если 0 - данные x, y, r не получилось достоверно расчитать.
    _length = 0  # в кадрах

    def __init__(self, doLogging = False):
        self.doLogging = doLogging

    def LoadVideo(self, path):
        self.inputPath = path
        vcap = cv2.VideoCapture(path)
        if vcap.isOpened():
            width = vcap.get(cv2.CAP_PROP_FRAME_WIDTH)
            height = vcap.get(cv2.CAP_PROP_FRAME_HEIGHT)
            self.EyetrackerFrameWidth = int(width)
            self.EyetrackerFrameHeight = int(height)
            self.EyetrackerFPS = vcap.get(cv2.CAP_PROP_FPS)
            self._length = int(vcap.get(cv2.CAP_PROP_FRAME_COUNT))
##            print('width, height, fps, total frames:', width, height, self.EyetrackerFPS, self._length)  # debug only
        vcap.release()

    def _DoLog(self, line):
        # запись лога
        if self.doLogging:
            path = os.curdir + '\\log_videoprocessor.txt'
            file1 = open(path, 'a')
            line = str(datetime.datetime.now()) + ' ' + line + '\n'
            file1.write(line)
            file1.close()

    def _bilateralFtr1D(self, y, sSpatial = 0.2, sIntensity = 1):
        """
        version: 1.0
        author: akshay
        date: 10/03/2016

        The equation of the bilateral filter is

                (       dx ^ 2       )       (         dI ^2        )
        F = exp (- ----------------- ) * exp (- ------------------- )
                (  sigma_spatial ^ 2 )       (  sigma_Intensity ^ 2 )
            ~~~~~~~~~~~~~~~~~~~~~~~~~~
            This is a guassian filter!
            dx - The 'geometric' distance between the 'center pixel' and the pixel
             to sample
        dI - The difference between the intensity of the 'center pixel' and
             the pixel to sample
        sigma_spatial and sigma_Intesity are constants. Higher values mean
        that we 'tolerate more' higher value of the distances dx and dI.

        Dependencies: numpy, scipy.ndimage.gaussian_filter1d

        calc gaussian kernel size as: filterSize = (2 * radius) + 1; radius = floor (2 * sigma_spatial)
        y - input data
        """
        # gaussian filter and parameters
        radius = np.floor (2 * sSpatial)
        filterSize = int((2 * radius) + 1)
        ftrArray = np.zeros(filterSize)
        ftrArray[int(radius)] = 1

        # Compute the Gaussian filter part of the Bilateral filter
        gauss = ndi.gaussian_filter1d(ftrArray, sSpatial)

        # 1d data dimensions
        width = y.size

        # 1d resulting data
        ret = np.zeros (width)

        for i in range(width):

            ## To prevent accessing values outside of the array
            # The left part of the lookup area, clamped to the boundary
            xmin = int(max (i - radius, 1));
            # How many columns were outside the image, on the left?
            dxmin = int(xmin - (i - radius));

            # The right part of the lookup area, clamped to the boundary
            xmax = int(min (i + radius, width));
            # How many columns were outside the image, on the right?
            dxmax = int((i + radius) - xmax);

            # The actual range of the array we will look at
            area = y [xmin:xmax]

            # The center position
            center = y [i]

            # The left expression in the bilateral filter equation
            # We take only the relevant parts of the matrix of the
            # Gaussian weights - we use dxmin, dxmax, dymin, dymax to
            # ignore the parts that are outside the image
            expS = gauss[(1+dxmin):(filterSize-dxmax)]

            # The right expression in the bilateral filter equation
            dy = y [xmin:xmax] - y [i]
            dIsquare = (dy * dy)
            expI = np.exp (- dIsquare / (sIntensity * sIntensity))

            # The bilater filter (weights matrix)
            F = expI * expS

            # Normalized bilateral filter
            Fnormalized = F / sum(F)

            # Multiply the area by the filter
            tempY = y [xmin:xmax] * Fnormalized

            # The resulting pixel is the sum of all the pixels in
            # the area, according to the weights of the filter
            # ret(i,j,R) = sum (tempR(:))
            ret[i] = sum (tempY)
        return ret


    def DoProcessing(self, videoprocessing_fastener = 1):
        # основной метод обработки.
        # videoprocessing_fastener - параметр для ускорения обработки ценой качества. Чем выше - тем быстрее.
        # Для повышения скорости снижается разрешение картинки и количество отслеживаемых особых точек.
        if self.EyetrackerFrameWidth == -1:
            self._DoLog('Couldn''t open ' + self.inputPath)
            print('Couldn''t open ' + self.inputPath)  # debug only
            return

        doSpeedup = videoprocessing_fastener != 1
        new_height = int(self.EyetrackerFrameHeight/videoprocessing_fastener)
        new_width = int(self.EyetrackerFrameWidth/videoprocessing_fastener)
        new_dimensions = (new_width, new_height)

        EyetrackerPixToFOV = self.EyetrackerHorizontalFOV/self.EyetrackerFrameWidth
        self._DoLog('start processing of ' + self.inputPath)
        cap = cv2.VideoCapture(self.inputPath)
        num_frames = int(cap.get(cv2.CAP_PROP_FRAME_COUNT))

        # если раскомментировать, то можно будет проверить,
        # вдруг мы этот файл уже обработали, полностью или частично, и продолжить обработку
        # if os.path.isfile(outputTxt):
        #     line = LastLine(outputTxt)
        #     prevFrameNo = int(line.split(';')[0]) - 1
        #     if (num_frames - prevFrameNo) < 10:  # на последние 10 кадров плевать
        #         self._DoLog('already processed ' + self.inputPath)
        #         return
        #     print('Skiping frames. Start processing from frame No ' + str(prevFrameNo))
        #     for i in range(0, prevFrameNo):
        #         _, prev_frame = cap.read()
        #         if i/1000 == round(i/1000):
        #             print(i)
        #     fileTxt = open(outputTxt, 'a')
        #     prevFrameNo = prevFrameNo
        # else:
        #     # файл ни разу не обрабатывался ранее
        #     prevFrameNo = 0
        #     fileTxt = open(outputTxt, 'w')
        #     fileTxt.write('FrameNo;eye_X;eye_Y;FlightInstr1_X;FlightInstr1_Y;FlightInstr1_W;FlightInstr1_H;FlightInstr2_X;FlightInstr2_Y;FlightInstr2_W;FlightInstr2_H;FlightInstr3_X;FlightInstr3_Y;FlightInstr3_W;FlightInstr3_H;FlightInstr4_X;FlightInstr4_Y;FlightInstr4_W;FlightInstr4_H;FrameShift_X;frameShift_Y;frameRotation_deg\n')
        #     fileTxt.write('1;-0;-0;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;0;0;0;\n')

        self._trajectory_x = [0] * self._length
        self._trajectory_y = [0] * self._length
        self._trajectory_r = [0] * self._length
        self._confidence = [0] * self._length
        self._confidence[0] = 1
        self._length = num_frames
        prevFrameNo = 0
        _, prev_frame = cap.read()  # читаем первый кадр
        prev_gray = cv2.cvtColor(prev_frame, cv2.COLOR_BGR2GRAY)

        if doSpeedup:  # тут даунсайз изображения
            prev_gray = cv2.resize(prev_gray, new_dimensions)

        for i in range(prevFrameNo+1, num_frames):
            """ # debug only
            if i == round(num_frames * 0.01):
                print(self.inputPath + ":  1% of processing done")
            if i == round(num_frames * 0.33):
                print(self.inputPath + ": 33% of processing done")
            if i == round(num_frames * 0.66):
                print(self.inputPath + ": 66% of processing done")
            """

            # Вычисляем оптический поток
            translation_x = 0
            translation_y = 0
            rotation_angle = 0
            confidence_est = 0
            tmpMaxCorners = int(200 / videoprocessing_fastener)

            prev_points = cv2.goodFeaturesToTrack(
                prev_gray,
                maxCorners=tmpMaxCorners,
                qualityLevel=0.1,
                minDistance=20,
                blockSize=4
            )

            success, curr_frame = cap.read()  # читаем очередной кадр
            if not success:
                self._DoLog('Error reading file ' + self.inputPath + ' |frame ' + str(i))
                break

            curr_gray = cv2.cvtColor(curr_frame, cv2.COLOR_BGR2GRAY)
            if doSpeedup:  # тут даунсайз изображения
                curr_gray = cv2.resize(curr_gray, new_dimensions)

            try:
                # вычисляем оптический поток
                curr_points, status, err = cv2.calcOpticalFlowPyrLK(
                    prev_gray,
                    curr_gray,
                    prev_points,
                    None,
                    2
                )
            except:
                # сюда попадаем в основном когда видео битое, и кадр представляет собой серый прямоугольник
                self._DoLog('another bad frame ' + str(i+2) + ' in ' + self.inputPath)
                self._trajectory_x[i] = self._trajectory_x[i-1]  # вставляем предыдущие значения
                self._trajectory_y[i] = self._trajectory_y[i-1]
                self._trajectory_r[i] = self._trajectory_r[i-1]
                self._confidence[i]   = 0  # в этом кадре мы совершенно не уверены
                continue

            assert prev_points.shape == curr_points.shape
            idx = np.where(status == 1)[0]
            prev_points = prev_points[idx]
            curr_points = curr_points[idx]

            if len(curr_points) > 0:
                # Оцениваем параметры аффинного преобразования между точками
                try:
                    matrix, _ = cv2.estimateAffine2D(prev_points, curr_points)
                    translation_x = matrix[0, 2]
                    translation_y = matrix[1, 2]
                    rotation_angle = (180/pi) * np.arctan2(matrix[1, 0], matrix[0, 0])
                    confidence_est = 1

                    # переводим из пикселей в градусы
                    translation_x = translation_x * EyetrackerPixToFOV
                    translation_y = translation_y * EyetrackerPixToFOV
                except:
                    self._DoLog('bad frame ' + str(i+2) + ' in ' + self.inputPath)

            if abs(rotation_angle) > 15:
                # слишком резким наклон головы быть не может, 450 градусов/сек - за гранью
                translation_x = 0
                translation_y = 0
                rotation_angle = 0
                confidence_est = 0

            if doSpeedup:
                translation_x  = translation_x * videoprocessing_fastener
                translation_y  = translation_y * videoprocessing_fastener
                rotation_angle = rotation_angle * videoprocessing_fastener

            self._trajectory_x[i] = translation_x + self._trajectory_x[i-1]  # записываем в массив, попутно переводя из относительных
            self._trajectory_y[i] = translation_y + self._trajectory_y[i-1]  # координат в абсолютные
            self._trajectory_r[i] = rotation_angle + self._trajectory_r[i-1]
            self._confidence[i] = confidence_est

            prev_gray = curr_gray

        cap.release()
        self._DoLog('finished processing of ' + self.inputPath)


    def DoPostprocessing(self, FilteringPower = 1):
        # FilteringPower характеризует силу постпроцессинга.
        # Значение 0 - это отсутствие фильтрации вообще.
        # Значение 1 означает, что будут вычислены медианные значения за первые
        # 60 секунд, и их вычтут из остальных величин.
        # Значение более 1 означает применение ещё и билатеральной фильтрации к
        # величинам x, y и r. Чем выше величина - тем сильнее фильтрация.
        if FilteringPower == 0:
          return

        if FilteringPower >= 1:
          time_sec = 60
          amount_of_frames = int(time_sec * self.EyetrackerFPS)
          n = min(self._length, amount_of_frames)
          tx = self._trajectory_x[1:n]
          ty = self._trajectory_y[1:n]
          tr = self._trajectory_r[1:n]

          med_x = st.median(tx)
          med_y = st.median(ty)
          med_r = st.median(tr)

          self._trajectory_x = [x - med_x for x in self._trajectory_x]
          self._trajectory_y = [y - med_y for y in self._trajectory_y]
          self._trajectory_r = [r - med_r for r in self._trajectory_r]

          if FilteringPower == 1:
            return

          # применяем билатеральную фильтрацию
          sigma_r = FilteringPower / 5
          sigma_i = FilteringPower - 1
          np_trajectory_x = self._bilateralFtr1D(np.asarray(self._trajectory_x), sigma_r, sigma_i)
          self._trajectory_x = np_trajectory_x.tolist()

          np_trajectory_y = self._bilateralFtr1D(np.asarray(self._trajectory_y), sigma_r, sigma_i)
          self._trajectory_y = np_trajectory_y.tolist()

          sigma_i = sigma_i/2
          np_trajectory_r = self._bilateralFtr1D(np.asarray(self._trajectory_r), sigma_r, sigma_i)
          self._trajectory_r = np_trajectory_r.tolist()


    def SaveToCSV(self, path):
        gaze_timestamp = 0
        fileTxt = open(path, 'w')
        fileTxt.write('gaze_timestamp,world_index,confidence,head_pos_x,head_pos_y,head_rotation,gaze_point_3d_x,gaze_point_3d_y,gaze_point_3d_z,eye_center0_3d_x,eye_center0_3d_y,eye_center0_3d_z,gaze_normal0_x,gaze_normal0_y,gaze_normal0_z,eye_center1_3d_x,eye_center1_3d_y,eye_center1_3d_z,gaze_normal1_x,gaze_normal1_y,gaze_normal1_z\n')
        for i in range(0, self._length):
           translation_x  = round(self._trajectory_x[i], 3)
           translation_y  = round(self._trajectory_y[i], 3)
           rotation_angle = round(self._trajectory_r[i], 3)
           confidence = self._confidence[i]

           gz_time = round(gaze_timestamp, 3)

           line = str(gz_time) + ',' + str(i+1) + ',' +  str(confidence) + ',' + str(translation_x) + ',' + str(translation_y) + ',' + str(rotation_angle) + ',\n'
           fileTxt.write(line)
           gaze_timestamp = gaze_timestamp + 1/self.EyetrackerFPS

        fileTxt.close()

if __name__ == '__main__':
    # Пример использования
    inputVideoPath = "video_test.avi"
    outputPath = inputVideoPath + '+alpha.csv'

    v = VideoProcessor(doLogging=True)
    v.EyetrackerHorizontalFOV = 82  # угол обзора камеры сцены айтрекера. Обычно есть в паспорте модели.
    v.LoadVideo(inputVideoPath)
    v.DoProcessing(5)  # рекомендуемое значение 1
    v.DoPostprocessing(5)  # рекомендуемое значение 1. Выше - для сильно зашумлённых данных.
    v.SaveToCSV(outputPath)

    print("Done!")



from abc import ABC, abstractmethod
# абстрактный класс, задающий основную логику для классов описывающих различные модели оценки
class Evaluator(ABC):
    @abstractmethod
    def evaluate(self, data):
        return False